@@ -3,7 +3,7 @@
 
 /* Initial coding by Rod Smith, January to February, 2009 */
 
-/* This program is copyright (c) 2009-2011 by Roderick W. Smith. It is distributed
+/* This program is copyright (c) 2009-2013 by Roderick W. Smith. It is distributed
   under the terms of the GNU GPL version 2, as detailed in the COPYING file. */
 
 #define __STDC_LIMIT_MACROS
@@ -701,10 +701,11 @@ int BasicMBRData::LBAtoCHS(uint64_t lba, uint8_t * chs) {
    return (retval);
 } // BasicMBRData::LBAtoCHS()
 
-// Look for overlapping partitions.
+// Look for overlapping partitions. Also looks for a couple of non-error
+// conditions that the user should be told about.
 // Returns the number of problems found
 int BasicMBRData::FindOverlaps(void) {
-   int i, j, numProbs = 0, numEE = 0;
+   int i, j, numProbs = 0, numEE = 0, ProtectiveOnOne = 0;
 
    for (i = 0; i < MAX_MBR_PARTS; i++) {
       for (j = i + 1; j < MAX_MBR_PARTS; j++) {
@@ -717,14 +718,17 @@ int BasicMBRData::FindOverlaps(void) {
       } // for (j...)
       if (partitions[i].GetType() == 0xEE) {
          numEE++;
-         if (partitions[i].GetStartLBA() != 1)
-            cout << "\nWarning: 0xEE partition doesn't start on sector 1. This can cause "
-                 << "problems\nin some OSes.\n";
+         if (partitions[i].GetStartLBA() == 1)
+            ProtectiveOnOne = 1;
       } // if
    } // for (i...)
+
    if (numEE > 1)
       cout << "\nCaution: More than one 0xEE MBR partition found. This can cause problems\n"
            << "in some OSes.\n";
+   if (!ProtectiveOnOne)
+      cout << "\nWarning: 0xEE partition doesn't start on sector 1. This can cause "
+           << "problems\nin some OSes.\n";
 
    return numProbs;
 } // BasicMBRData::FindOverlaps()
@@ -903,7 +907,7 @@ int BasicMBRData::SpaceBeforeAllLogicals(void) {
 } // BasicMBRData::SpaceBeforeAllLogicals()
 
 // Returns 1 if the partitions describe a legal layout -- all logicals
-// are contiguous and have at least one preceding empty partitions,
+// are contiguous and have at least one preceding empty sector,
 // the number of primaries is under 4 (or under 3 if there are any
 // logicals), there are no overlapping partitions, etc.
 // Does NOT assume that primaries are numbered 1-4; uses the
@@ -921,6 +925,18 @@ int BasicMBRData::IsLegal(void) {
    return allOK;
 } // BasicMBRData::IsLegal()
 
+// Returns 1 if the 0xEE partition in the protective/hybrid MBR is marked as
+// active/bootable.
+int BasicMBRData::IsEEActive(void) {
+   int i, IsActive = 0;
+
+   for (i = 0; i < MAX_MBR_PARTS; i++) {
+      if ((partitions[i].GetStatus() & 0x80) && (partitions[i].GetType() == 0xEE))
+         IsActive = 1;
+   }
+   return IsActive;
+} // BasicMBRData::IsEEActive()
+
 // Finds the next in-use partition, starting with start (will return start
 // if it's in use). Returns -1 if no subsequent partition is in use.
 int BasicMBRData::FindNextInUse(int start) {
@@ -1044,6 +1060,7 @@ int BasicMBRData::MakeBiggestPart(int i, int type) {
    uint64_t selectedSegment = UINT64_C(0); // location of largest segment
    uint64_t selectedSize = UINT64_C(0); // size of largest segment in blocks
    int found = 0;
+   string anything;
 
    do {
       firstBlock = FindFirstAvailable(start);
@@ -1363,6 +1380,9 @@ uint64_t BasicMBRData::FindFirstAvailable(uint64_t start) {
    uint64_t i;
    int firstMoved;
 
+   if ((start >= (UINT32_MAX - 1)) || (start >= (diskSize - 1)))
+      return 0;
+
    first = start;
 
    // ...now search through all partitions; if first is within an
@@ -1532,6 +1552,7 @@ int BasicMBRData::DoMenu(const string& prompt) {
       cout << prompt;
       switch (ReadString()[0]) {
          case '\0':
+            goOn = cin.good();
             break;
          case 'a': case 'A':
             num = GetNumber(1, MAX_MBR_PARTS, 1, "Toggle active flag for partition: ") - 1;
