@@ -19,6 +19,7 @@
 #include <string.h>
 #include <string>
 #include <stdint.h>
+#include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -73,7 +74,7 @@ int DiskIO::OpenForRead(void) {
          if (fstat64(fd, &st) == 0) {
             if (S_ISDIR(st.st_mode))
                cerr << "The specified path is a directory!\n";
-#ifndef __FreeBSD__
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
             else if (S_ISCHR(st.st_mode))
                cerr << "The specified path is a character device!\n";
 #endif
@@ -104,6 +105,7 @@ int DiskIO::OpenForWrite(void) {
 #ifdef __APPLE__
    // MacOS X requires a shared lock under some circumstances....
    if (fd < 0) {
+      cerr << "Warning: Devices opened with shared lock will not have their\npartition table automatically reloaded!\n";
       fd = open(realFilename.c_str(), O_WRONLY | O_SHLOCK);
    } // if
 #endif
@@ -132,6 +134,9 @@ void DiskIO::Close(void) {
 // (512). If the disk can't be opened at all, return a value of 0.
 int DiskIO::GetBlockSize(void) {
    int err = -1, blockSize = 0;
+#ifdef __sun__
+   struct dk_minfo minfo;
+#endif
 
    // If disk isn't open, try to open it....
    if (!isOpen) {
@@ -142,6 +147,11 @@ int DiskIO::GetBlockSize(void) {
 #ifdef __APPLE__
       err = ioctl(fd, DKIOCGETBLOCKSIZE, &blockSize);
 #endif
+#ifdef __sun__
+      err = ioctl(fd, DKIOCGMEDIAINFO, &minfo);
+      if (err == 0)
+          blockSize = minfo.dki_lbsize;
+#endif
 #if defined (__FreeBSD__) || defined (__FreeBSD_kernel__)
       err = ioctl(fd, DIOCGSECTORSIZE, &blockSize);
 #endif
@@ -217,13 +227,17 @@ int DiskIO::DiskSync(void) {
 
    if (isOpen) {
       sync();
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__sun__)
       cout << "Warning: The kernel may continue to use old or deleted partitions.\n"
            << "You should reboot or remove the drive.\n";
                /* don't know if this helps
                * it definitely will get things on disk though:
                * http://topiks.org/mac-os-x/0321278542/ch12lev1sec8.html */
+#ifdef __sun__
+      i = ioctl(fd, DKIOCFLUSHWRITECACHE);
+#else
       i = ioctl(fd, DKIOCSYNCHRONIZECACHE);
+#endif
       platformFound++;
 #endif
 #if defined (__FreeBSD__) || defined (__FreeBSD_kernel__)
@@ -377,6 +391,9 @@ uint64_t DiskIO::DiskSize(int *err) {
    off_t bytes = 0; // size in bytes
    struct stat64 st;
    int platformFound = 0;
+#ifdef __sun__
+   struct dk_minfo minfo;
+#endif
 
    // If disk isn't open, try to open it....
    if (!isOpen) {
@@ -393,6 +410,12 @@ uint64_t DiskIO::DiskSize(int *err) {
       *err = ioctl(fd, DKIOCGETBLOCKCOUNT, &sectors);
       platformFound++;
 #endif
+#ifdef __sun__
+      *err = ioctl(fd, DKIOCGMEDIAINFO, &minfo);
+      if (*err == 0)
+          sectors = minfo.dki_capacity;
+      platformFound++;
+#endif
 #if defined (__FreeBSD__) || defined (__FreeBSD_kernel__)
       *err = ioctl(fd, DIOCGMEDIASIZE, &bytes);
       long long b = GetBlockSize();
