@@ -3,7 +3,7 @@
 
 /* By Rod Smith, initial coding January to February, 2009 */
 
-/* This program is copyright (c) 2009-2011 by Roderick W. Smith. It is distributed
+/* This program is copyright (c) 2009-2013 by Roderick W. Smith. It is distributed
   under the terms of the GNU GPL version 2, as detailed in the COPYING file. */
 
 #define __STDC_LIMIT_MACROS
@@ -259,6 +259,14 @@ int GPTData::Verify(void) {
            << "The 'e' option on the experts' menu may fix this problem.\n";
    } // if
 
+   if ((mainHeader.lastUsableLBA >= diskSize) || (mainHeader.lastUsableLBA > mainHeader.backupLBA)) {
+      problems++;
+      cout << "\nProblem: GPT claims the disk is larger than it is! (Claimed last usable\n"
+           << "sector is " << mainHeader.lastUsableLBA << ", but backup header is at\n"
+           << mainHeader.backupLBA << " and disk size is " << diskSize << " sectors.\n"
+           << "The 'e' option on the experts' menu will probably fix this problem\n";
+   }
+
    // Check for overlapping partitions....
    problems += FindOverlaps();
 
@@ -271,6 +279,15 @@ int GPTData::Verify(void) {
    // Check for MBR-specific problems....
    problems += VerifyMBR();
 
+   // Check for a 0xEE protective partition that's marked as active....
+   if (protectiveMBR.IsEEActive()) {
+      cout << "\nWarning: The 0xEE protective partition in the MBR is marked as active. This is\n"
+           << "technically a violation of the GPT specification, and can cause some EFIs to\n"
+           << "ignore the disk, but it is required to boot from a GPT disk on some BIOS-based\n"
+           << "computers. You can clear this flag by creating a fresh protective MBR using\n"
+           << "the 'n' option on the experts' menu.\n";
+   }
+
    // Verify that partitions don't run into GPT data areas....
    problems += CheckGPTSize();
 
@@ -556,7 +573,7 @@ int GPTData::FindHybridMismatches(void) {
          mbrFirst = (uint64_t) protectiveMBR.GetFirstSector(i);
          mbrLast = mbrFirst + (uint64_t) protectiveMBR.GetLength(i) - UINT64_C(1);
          do {
-            if ((partitions[j].GetFirstLBA() == mbrFirst) &&
+            if ((j < numParts) && (partitions[j].GetFirstLBA() == mbrFirst) &&
                 (partitions[j].GetLastLBA() == mbrLast) && (partitions[j].IsUsed()))
                found = 1;
             j++;
@@ -1018,6 +1035,21 @@ int GPTData::SaveGPTData(int quiet) {
       } // if/else quiet
    } // if
 
+   if ((mainHeader.lastUsableLBA >= diskSize) || (mainHeader.lastUsableLBA > mainHeader.backupLBA)) {
+      if (quiet == 0) {
+         cout << "Warning! The claimed last usable sector is incorrect! Do you want to correct\n"
+              << "this problem? ";
+         if (GetYN() == 'Y') {
+            MoveSecondHeaderToEnd();
+            cout << "Have adjusted the second header and last usable sector value.\n";
+         } else {
+            cout << "Have not corrected the problem. Strange problems may occur in the future!\n";
+         } // if correction requested
+      } else { // go ahead and do correction automatically
+         MoveSecondHeaderToEnd();
+      } // if/else quiet
+   } // if
+
    // Check for overlapping or insane partitions....
    if ((FindOverlaps() > 0) || (FindInsanePartitions() > 0)) {
       allOK = 0;
@@ -1379,10 +1411,10 @@ void GPTData::DisplayGPTData(void) {
 
 // Show detailed information on the specified partition
 void GPTData::ShowPartDetails(uint32_t partNum) {
-   if (!IsFreePartNum(partNum)) {
+   if ((partNum < numParts) && !IsFreePartNum(partNum)) {
       partitions[partNum].ShowDetails(blockSize);
    } else {
-      cout << "Partition #" << partNum + 1 << " does not exist.";
+      cout << "Partition #" << partNum + 1 << " does not exist.\n";
    } // if
 } // GPTData::ShowPartDetails()
 
@@ -1406,12 +1438,14 @@ WhichToUse GPTData::UseWhichPartitions(void) {
 
    if ((state == gpt_invalid) && ((mbrState == mbr) || (mbrState == hybrid))) {
       cout << "\n***************************************************************\n"
-           << "Found invalid GPT and valid MBR; converting MBR to GPT format.\n";
+           << "Found invalid GPT and valid MBR; converting MBR to GPT format\n"
+           << "in memory. ";
       if (!justLooking) {
-         cout << "\aTHIS OPERATION IS POTENTIALLY DESTRUCTIVE! Exit by typing 'q' if\n"
-              << "you don't want to convert your MBR partitions to GPT format!\n";
+         cout << "\aTHIS OPERATION IS POTENTIALLY DESTRUCTIVE! Exit by\n"
+              << "typing 'q' if you don't want to convert your MBR partitions\n"
+              << "to GPT format!";
       } // if
-      cout << "***************************************************************\n\n";
+      cout << "\n***************************************************************\n\n";
       which = use_mbr;
    } // if
 
@@ -1718,7 +1752,7 @@ uint32_t GPTData::CreatePartition(uint32_t partNum, uint64_t startSector, uint64
          if (FindLastInFree(startSector) >= endSector) {
             partitions[partNum].SetFirstLBA(startSector);
             partitions[partNum].SetLastLBA(endSector);
-            partitions[partNum].SetType(DEFAULT_TYPE);
+            partitions[partNum].SetType(DEFAULT_GPT_TYPE);
             partitions[partNum].RandomizeUniqueGUID();
          } else retval = 0; // if free space until endSector
       } else retval = 0; // if startSector is free
@@ -2065,7 +2099,7 @@ uint64_t GPTData::FindFirstInLargest(void) {
 } // GPTData::FindFirstInLargest()
 
 // Find the last available block on the disk.
-// Returns 0 if there are no available partitions
+// Returns 0 if there are no available sectors
 uint64_t GPTData::FindLastAvailable(void) {
    uint64_t last;
    uint32_t i;
@@ -2196,12 +2230,12 @@ void GPTData::SetAlignment(uint32_t n) {
 // sector size), but not by the previously-located alignment value, then the
 // alignment value is adjusted down. If the computed alignment is less than 8
 // and the disk is bigger than SMALLEST_ADVANCED_FORMAT, resets it to 8. This
-// is a safety measure for WD Advanced Format and similar drives. If no partitions
-// are defined, the alignment value is set to DEFAULT_ALIGNMENT (2048) (or an
+// is a safety measure for Advanced Format drives. If no partitions are
+// defined, the alignment value is set to DEFAULT_ALIGNMENT (2048) (or an
 // adjustment of that based on the current sector size). The result is that new
 // drives are aligned to 2048-sector multiples but the program won't complain
 // about other alignments on existing disks unless a smaller-than-8 alignment
-// is used on big disks (as safety for WD Advanced Format drives).
+// is used on big disks (as safety for Advanced Format drives).
 // Returns the computed alignment value.
 uint32_t GPTData::ComputeAlignment(void) {
    uint32_t i = 0, found, exponent = 31;
@@ -2298,32 +2332,38 @@ int GPTData::ManageAttributes(int partNum, const string & command, const string
    int retval = 0;
    Attributes theAttr;
 
-   if (command == "show") {
-      ShowAttributes(partNum);
-   } else if (command == "get") {
-      GetAttribute(partNum, bits);
+   if (partNum >= (int) numParts) {
+      cerr << "Invalid partition number (" << partNum + 1 << ")\n";
+      retval = -1;
    } else {
-      theAttr = partitions[partNum].GetAttributes();
-      if (theAttr.OperateOnAttributes(partNum, command, bits)) {
-         partitions[partNum].SetAttributes(theAttr.GetAttributes());
-         retval = 1;
+      if (command == "show") {
+         ShowAttributes(partNum);
+      } else if (command == "get") {
+         GetAttribute(partNum, bits);
       } else {
-         retval = -1;
-      } // if/else
-   } // if/elseif/else
+         theAttr = partitions[partNum].GetAttributes();
+         if (theAttr.OperateOnAttributes(partNum, command, bits)) {
+            partitions[partNum].SetAttributes(theAttr.GetAttributes());
+            retval = 1;
+         } else {
+            retval = -1;
+         } // if/else
+      } // if/elseif/else
+   } // if/else invalid partition #
 
    return retval;
 } // GPTData::ManageAttributes()
 
 // Show all attributes for a specified partition....
 void GPTData::ShowAttributes(const uint32_t partNum) {
-   if (partitions[partNum].IsUsed())
+   if ((partNum < numParts) && partitions[partNum].IsUsed())
       partitions[partNum].ShowAttributes(partNum);
 } // GPTData::ShowAttributes
 
 // Show whether a single attribute bit is set (terse output)...
 void GPTData::GetAttribute(const uint32_t partNum, const string& attributeBits) {
-   partitions[partNum].GetAttributes().OperateOnAttributes(partNum, "get", attributeBits);
+   if (partNum < numParts)
+      partitions[partNum].GetAttributes().OperateOnAttributes(partNum, "get", attributeBits);
 } // GPTData::GetAttribute
 
 
