@@ -134,7 +134,7 @@ public:
    // Convert between GPT and other formats
    virtual WhichToUse UseWhichPartitions(void);
    void XFormPartitions(void);
-   virtual int XFormDisklabel(uint32_t partNum);
+   int XFormDisklabel(uint32_t partNum);
    int XFormDisklabel(BSDData* disklabel);
    int OnePartToMBR(uint32_t gptPart, int mbrPart); // add one partition to MBR. Returns 1 if successful
 
@@ -183,7 +183,7 @@ public:
    int IsFree(uint64_t sector, uint32_t *partNum = NULL);
    int IsFreePartNum(uint32_t partNum);
    int IsUsedPartNum(uint32_t partNum);
-   
+
    // Change how functions work, or return information on same
    void SetAlignment(uint32_t n);
    uint32_t ComputeAlignment(void); // Set alignment based on current partitions
