@@ -1,7 +1,7 @@
 /*
  *    Implementation of GPTData class derivative with curses-based text-mode
  *    interaction
- *    Copyright (C) 2011 Roderick W. Smith
+ *    Copyright (C) 2011-2013 Roderick W. Smith
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ GPTDataCurses::GPTDataCurses(void) {
    currentSpaceNum = -1;
    whichOptions = ""; // current set of options
    currentKey = 'b'; // currently selected option
+   displayType = USE_CURSES;
 } // GPTDataCurses constructor
 
 GPTDataCurses::~GPTDataCurses(void) {
@@ -219,7 +220,9 @@ void GPTDataCurses::IdentifySpaces(void) {
 Space* GPTDataCurses::ShowSpace(int spaceNum, int lineNum) {
    Space *space;
    int i = 0;
+#ifdef USE_UTF16
    char temp[40];
+#endif
 
    space = firstSpace;
    while ((space != NULL) && (i < spaceNum)) {
@@ -271,10 +274,16 @@ int GPTDataCurses::DisplayParts(int selected) {
    for (i = pageNum * numToShow; i <= (pageNum + 1) * numToShow - 1; i++) {
       if (i < numSpaces) { // real space; show it
          if (i == selected) {
-            attron(A_REVERSE);
             currentSpaceNum = i;
-            currentSpace = ShowSpace(i, lineNum++);
-            attroff(A_REVERSE);
+            if (displayType == USE_CURSES) {
+               attron(A_REVERSE);
+               currentSpace = ShowSpace(i, lineNum++);
+               attroff(A_REVERSE);
+            } else {
+               currentSpace = ShowSpace(i, lineNum);
+               move(lineNum++, 0);
+               printw(">");
+            }
             DisplayOptions(i);
             retval = selected;
          } else {
@@ -308,7 +317,9 @@ void GPTDataCurses::DeletePartition(int partNum) {
 // Displays information on the specified partition
 void GPTDataCurses::ShowInfo(int partNum) {
    uint64_t size;
+#ifdef USE_UTF16
    char temp[NAME_SIZE / 2 + 1];
+#endif
 
    clear();
    move(2, (COLS - 29) / 2);
@@ -726,8 +737,9 @@ void GPTDataCurses::DrawMenu(void) {
    string drive="Disk Drive: ";
    drive += device;
    ostringstream size;
+
    size << "Size: " << diskSize << ", " << BytesToIeee(diskSize, blockSize);
-   
+
    clear();
    move(0, (COLS - title.length()) / 2);
    printw(title.c_str());
@@ -800,7 +812,7 @@ void ShowTypes(void) {
 
    def_prog_mode();
    endwin();
-   tempType.ShowAllTypes();
+   tempType.ShowAllTypes(LINES - 3);
    cout << "\nPress the <Enter> key to continue: ";
    cin.get(junk);
    reset_prog_mode();
