@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2010-2011  <Roderick W. Smith>
+    Copyright (C) 2010-2013  <Roderick W. Smith>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <stdint.h>
 #include <limits.h>
 #include <iostream>
+#include <fstream>
 #include <sstream>
 #include <cstdio>
 #include "attributes.h"
@@ -131,7 +132,7 @@ int GPTDataTextUI::XFormDisklabel(void) {
       numDone = GPTData::XFormDisklabel(partNum);
 
    return numDone;
-} // GPTData::XFormDisklabel(int i)
+} // GPTData::XFormDisklabel(void)
 
 
 /*********************************************************************
@@ -194,6 +195,7 @@ void GPTDataTextUI::CreatePartition(void) {
        (firstFreePart < numParts)) {
       lastBlock = FindLastAvailable();
       firstInLargest = FindFirstInLargest();
+      Align(&firstInLargest);
 
       // Get partition number....
       do {
@@ -450,7 +452,7 @@ void GPTDataTextUI::MakeHybrid(void) {
       if (eeFirst == 'Y') {
          hybridMBR.AddPart(0, hybridPart);
       } else {
-         hybridMBR.AddPart(3, hybridPart);
+         hybridMBR.AddPart(numPartsToCvt, hybridPart);
       } // else
       hybridMBR.SetHybrid();
 
@@ -508,11 +510,12 @@ void GPTDataTextUI::MainMenu(string filename) {
    int goOn = 1;
    PartType typeHelper;
    uint32_t temp1, temp2;
-   
+
    do {
       cout << "\nCommand (? for help): ";
       switch (ReadString()[0]) {
          case '\0':
+            goOn = cin.good();
             break;
          case 'b': case 'B':
             cout << "Enter backup filename to save: ";
@@ -608,6 +611,7 @@ void GPTDataTextUI::RecoveryMenu(string filename) {
       cout << "\nRecovery/transformation command (? for help): ";
       switch (ReadString()[0]) {
          case '\0':
+            goOn = cin.good();
             break;
          case 'b': case 'B':
             RebuildMainHeader();
@@ -649,7 +653,7 @@ void GPTDataTextUI::RecoveryMenu(string filename) {
                } // if
             } else {
                MakeProtectiveMBR();
-               SetGPTSize(numParts);
+               SetGPTSize(numParts, 0);
                cout << "Note: New protective MBR created\n\n";
             } // if/else
             break;
@@ -733,6 +737,7 @@ void GPTDataTextUI::ExpertsMenu(string filename) {
       cout << "\nExpert command (? for help): ";
       switch (ReadString()[0]) {
          case '\0':
+            goOn = cin.good();
             break;
          case 'a': case 'A':
             if (GetPartRange(&temp1, &temp2) > 0)
