@@ -3,7 +3,7 @@
 // Primarily by Rod Smith, February 2009, but with a few functions
 // copied from other sources (see attributions below).
 
-/* This program is copyright (c) 2009 by Roderick W. Smith. It is distributed
+/* This program is copyright (c) 2009-2013 by Roderick W. Smith. It is distributed
   under the terms of the GNU GPL version 2, as detailed in the COPYING file. */
 
 #define __STDC_LIMIT_MACROS
@@ -38,6 +38,8 @@ string ReadString(void) {
    string inString;
 
    getline(cin, inString);
+   if (!cin.good())
+      exit(5);
    return inString;
 } // ReadString()
 
@@ -54,6 +56,8 @@ int GetNumber(int low, int high, int def, const string & prompt) {
       do {
          cout << prompt;
          cin.getline(line, 255);
+         if (!cin.good())
+            exit(5);
          num = sscanf(line, "%d", &response);
          if (num == 1) { // user provided a response
             if ((response < low) || (response > high))
@@ -98,6 +102,8 @@ uint64_t GetSectorNum(uint64_t low, uint64_t high, uint64_t def, uint64_t sSize,
    do {
       cout << prompt;
       cin.getline(line, 255);
+      if (!cin.good())
+         exit(5);
       response = IeeeToInt(line, sSize, low, high, def);
    } while ((response < low) || (response > high));
    return response;
